pub enum VerificationMethod {
DidKey(Did),
DidUrl(Did),
}Expand description
A verification method most typically is a public key (via did:key), or a Did Url
that links to a verification method in a different Did Document.
Variants
DidKey(Did)
A did:key. This does not include the fragment suffix, to save space.
DidUrl(Did)
A reference to a verification method in a remote Did Document. Any method other
than did:key can be used.
DidUrls allow the use of verification methods that are controlled by third parties or with alternative did methods such as did:web. By referencing external Dids, users can use more convenient third party services while retaining their ability for credible exit.
Implementations